#!/bin/bash
VER="1.1"

###############################################################
# Group Delete 1.1 by Turranius                               #
# Script to either set flag 6 on a whole group, or to totally #
# delete all users in a group, and the group itself.          #
####                                                       ####
# Installation:                                               #
# Copy grpkill.sh to /glftpd/bin (or where ever you have it). #
# Make it executable.                                         #
# Edit the settings below so it fits your system.             #
# Create the backup folder if you have BACKUP to TRUE         #
# Add it to /etc/glftpd.conf as a custom script:              #
# site_cmd GRPKILL  EXEC  /bin/grpkill.sh                     #
# custom-grpkill  1                                           #
# Run 'site grpkill' from site for instructions.              #
#                                                             #
# Backed up users and group/passwd files will get the         #
# extension .MonthDayHourMinute (passwd.06271130).            #
#                                                             #
###############################################################
# History:                                                    #
# 1.1 : Added option to backup userfiles and passwd/group     #
#       files.                                                #
#       Added option to write to the syslog file, pretty much #
#       like it always does when you del users or groups.     #
#       FIX: Now it actually deletes the user files, and the  #
#       default paths are adjusted to fit glftpd's chrooted   #
#       enviroment. I was pretty tired when I did 1.0 :)      #
# 1.0 : Initial release.                                      #
###############################################################

PASSWD=/etc/passwd                 # Path to the passwd file
GROUPFILE=/etc/group               # Path to the group file
USERPATH=/ftp-data/users           # Path to the users folder
USERFILETMP=/tmp/grpkill.tmp       # Temp file for delling users. Needed.
SYSLOG="TRUE"                      # Log to sysop.log? TRUE/FALSE
SYSOPLOG=/ftp-data/logs/sysop.log  # Path to the sysop.log file, if SYSLOG is TRUE
BACKUP="TRUE"                      # Back up users and passwd/group? TRUE/FALSE
BACKUPFOLDER=/ftp-data/back        # Folder to backup users and passwd/group, if BACKUP is TRUE


#################################################################
# No changes below here                                         #
#################################################################

## Check that the files exist

if [ "$BACKUP" = "TRUE" ]; then
  if [ -e "$BACKUPFOLDER" ]; then
    USERP=OK
  else
    echo "Cant seem to find the $BACKUPFOLDER folder. Check your settings."
    exit 0
  fi
touch $BACKUPFOLDER/writetest.tmp
  if [ -e "$BACKUPFOLDER/writetest.tmp" ]; then 
    rm -f $BACKUPFOLDER/writetest.tmp
  else
    echo "Cant create files in $BACKUPFOLDER. Change permissions on the folder."
    exit 0
  fi
fi

if [ "$SYSLOG" = "TRUE" ]; then
  if [ -e "$SYSOPLOG" ]; then
    USERP=OK
  else
    echo "Cant seem to find the $SYSOPLOG folder. Check your settings."
    exit 0
  fi
fi

touch $USERFILETMP
if [ -e "$USERFILETMP" ]; then
  rm -rf $USERFILETMP
else
  echo "Cant seem to find the $USERFILETMP folder. Check your settings."
  exit 0
fi

if [ -e "$GROUPFILE" ]; then
  USERP=OK
else
  echo "Cant seem to find the $GROUPFILE folder. Check your settings."
  exit 0
fi

if [ -e "$PASSWD" ]; then
  USERP=OK
else
  echo "Cant seem to find the $PASSWD file. Check your settings."
  exit 0
fi

if [ -e "$USERPATH" ]; then
  USERP=OK
else
  echo "Cant seem to find the $USERPATH folder. Check your settings."
  exit 0
fi

if [ "$1" = "" ]; then
  echo "Groupkiller $VER by Turranius                           "
  echo "Usage:                                                  "
  echo "site grpkill <groupname> purge <test>                   "
  echo "                                                        " 
  echo "Leaving purge out will simply put the 6 flag on         "
  echo "all the users in that group. Otherwise, if you          "
  echo "do use purge, the users and the group will be deleted.  "
  echo "Beware. It does not matter to this command if the user  "
  echo "is in multiple groups. If he IS in the group you specify"
  echo "he will be deleted regardless.                          "
  echo "                                                        "
  echo "Doing 'test' as the last argument will NOT delete any   "
  echo "users, but it will say it will (Just so you can see     "
  echo "what would have happened).                              "
  echo "                                                        "
  exit 0
fi 

## Check if group exists.
for i in `cat $GROUPFILE | awk -F":" '{print $1}'`
do
  if [ "$EXISTS" != "YES" ]; then
    if [ "$i" = "$1" ]; then
      EXIST="YES"
    fi
  fi
done
if [ "$EXIST" != "YES" ]; then
  echo "$1 does not exist."
  exit 0
fi

if [ "$SYSLOG" = "TRUE" ]; then
  SYSLOGDATE="$( date +%a" "%b" "%d" "%T" "%Y" [grpkill ]" )"
fi
DATE="$(date +%m%d%I%M)"

cd $USERPATH

for i in `ls -f -A | egrep -v 'default.user|glftpd|backup|disabled'`
do
  GROUP="$(cat $i | grep -w GROUP | awk '{printf $2 " "}')"
  for eachgroup in $GROUP; do
    if [ "$MATCH" != "YEPP" ]; then
      if [ "$eachgroup" = "$1" ]; then
        if [ "$2" != "purge" ]; then
          ## Setting flag 6
          if [ "$BACKUP" = "TRUE" ]; then
            cp -f $USERPATH/$i $BACKUPFOLDER/$i.$DATE
          fi
          FLAGS="$(cat $USERPATH/$i | grep -w FLAGS | awk -F" " '{print $2}')"
          CHECK="$( echo $FLAGS | grep 6 )"
          if [ "$CHECK" != "" ]; then
            echo "$i - Already deleted" 
          else
            if [ "$2" = "test" ]; then
               echo "$i - Getting flag 6 (test mode)"
            else
              sed -e "s/^FLAGS $FLAGS.*/FLAGS "$FLAGS"6/" $USERPATH/$i > $USERFILETMP
              mv -f $USERFILETMP $USERPATH/$i
              echo "$i - Getting flag 6"
              if [ "$SYSLOG" = "TRUE" ]; then
                echo "$SYSLOGDATE '$USER' deleted user '$i'." >> $SYSOPLOG
              fi
            fi
          fi
        else
          ## Permanently killing users.
          USERLINE="$( cat $PASSWD | grep -w $i )"
          if [ "$GRPDELLED" != "YEPP" ]; then
            if [ "$BACKUP" = "TRUE" ]; then
              cp -f $PASSWD $BACKUPFOLDER/passwd.$DATE
              cp -f $GROUPFILE $BACKUPFOLDER/group.$DATE
            fi 
            GROUPLINE="$( cat $GROUPFILE | grep -w $1 )"
            if [ "$3" != "" ]; then
              echo "Deleting $1 from group file (test mode)"
              GRPDELLED="YEPP"
            else
              if [ "$SYSLOG" = "TRUE" ]; then
                echo "$SYSLOGDATE '$USER' deleted group '$1'." >> $SYSOPLOG
              fi
              cat $GROUPFILE | grep -v "$GROUPLINE" > $USERFILETMP
              mv -f $USERFILETMP $GROUPFILE            
              GRPDELLED="YEPP"
              echo "Deleting $1 from group file"
            fi
          fi
          if [ "$3" != "" ]; then
            echo "$i - Deleting permanently! (test mode)"
          else
            if [ "$BACKUP" = "TRUE" ]; then
              cp -f $USERPATH/$i $BACKUPFOLDER/$i.$DATE
            fi
            if [ "$SYSLOG" = "TRUE" ]; then
              echo "$SYSLOGDATE '$USER' deleted user '$i'." >> $SYSOPLOG
              echo "$SYSLOGDATE '$USER' purged '$i'." >> $SYSOPLOG
            fi
            echo "$i - Deleting permanently!"
            rm -f $USERPATH/$i
            cat $PASSWD | grep -v "$USERLINE" > $USERFILETMP
            mv -f $USERFILETMP $PASSWD
          fi
        fi
      fi
    fi
  done
done
